# -*- coding: iso-8859-1 -*-
#
# $Id: PropertysheetInstaller.py 19745 2006-02-24 05:40:36Z encolpe $
#
# *************************************************************
# ** (Un)installer for a property sheet in portal_properties **
# *************************************************************
#
# Example:
#
#     ...
#     myProperties = (
#         ('title', 'A specific property sheet', 'string'),
#         ('myValue', 100, 'int'))
#     psi = PropertysheetInstaller('my_propertysheet', myProperties)
#     ...
#
# Note: If the 'title' string property is not provided, a title with the
# id's value is provided.
#

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

from OFS.ObjectManager import BadRequestException

from utils import InstallerBase

__all__ = ('PropertysheetInstaller',)

class PropertysheetInstaller(InstallerBase):
    """(Un)installer for a property sheet in portal_properties
    """

    _installerTitle = "Propertysheet in portal properties"


    def __init__(self, sheet_name, properties, **kw):
        """Constructor
        @param sheet_name: the name as appearing in portal_properties
        @param properties: a sequence of (id, value, type) tuples as expected by manage_addProperty
        @param kw: override defaults like stop_on_error attribute
        """
        self.sheet_name = sheet_name
        self.properties = properties
        for k, v in kw.items():
            setattr(self, k, v)
        return


    def doInstall(self, context):
        """Adds the property sheet
        @param context: an InstallationContext object
        """
        title = self.sheet_name
        for id, value, tipe in self.properties:
            if id == 'title' and tipe == 'string':
                title = value
        try:
            context.portal_properties.addPropertySheet(self.sheet_name, title=title)
            context.logInfo("Created '%s' property sheet" % self.sheet_name)
        except BadRequestException, e:
            context.logWarning("'%s' property sheet is already in portal properties, skipped"
                               % self.sheet_name)
        else:
            propertySheet = getattr(context.portal_properties, self.sheet_name)
            for id, value, tipe in self.properties:
                if id != 'title':
                    propertySheet.manage_addProperty(id, value, tipe)
            context.logInfo("Added property sheet '%s' to portal_properties" % self.sheet_name)
        return


    def doUninstall(self, context):
        """Removing the property sheet
        @param context: an InstallationContext object
        """
        try:
            context.portal_properties.manage_delObjects(ids=self.sheet_name)
            context.logInfo("Removed property sheet '%s' from portal properties" % self.sheet_name)
        except BadRequestException, e:
            context.logWarning("Property sheet '%s' already removed from portal properties" % self.sheet_name)
        return
