# -*- coding: iso-8859-1 -*-
#
# $Id: TypeInstaller.py 19745 2006-02-24 05:40:36Z encolpe $
#
#
# *****************************************************
# ** (Un)installation of new types in the types tool **
# *****************************************************
#
# Example:
#
#     from Products.MyProduct import MyType1, MyType2, MyType3
#     ...
#     # Serveral types in one TypeInstaller object
#     ti1 = TypeInstaller((MyType1.factory_type_information,
# MyType2.factory_type_information))
#     # Only one type in one TypeInstaller object
#     ti2 = TypeInstaller(MyType3.factory_type_information)
#     ...

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

from Products.CMFCore.TypesTool import ContentFactoryMetadata

from utils import InstallerBase

__all__ = ('TypeInstaller',)

class TypeInstaller(InstallerQISupportBase):

    _installerTitle = "content type in portal types"


    def __init__(self, fti, **kw):
        """Constructor
        @param fti: a (sequence of) usual factory type information structure(s)
        @param kw: override defaults like stop_on_error attribute
        """
        if type(fti) == type({}):
            fti = [fti]
        self.fti = tuple(fti)
        for k, v in kw.items():
            setattr(self, k, v)
        return

    def doInstall(self, context):
        """Registers the types in portal_types
        @param context: InstallationContext object
        """
        actual_type_ids = context.portal_types.objectIds()
        for fti in self.fti:
            fti_id = fti['id']
            if fti_id in actual_type_ids:
                context.logWarning("Type %s already installed in portal types, skipping" %
                                   fti_id)
            else:
                cfm = apply(ContentFactoryMetadata, (), fti)
                context.portal_types._setObject(fti_id, cfm)
                context.logInfo("Type %s installed in portal types" % fti_id)
        return
